package com.winhc.data.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.OSS;
import com.aliyun.oss.model.GetObjectRequest;
import com.aliyun.oss.model.OSSObject;
import com.winhc.data.DataPull;
import com.winhc.data.utils.BaseUtils;
import com.winhc.data.utils.Common;
import com.winhc.data.utils.OssUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.concurrent.BlockingQueue;

/**
 * @author π
 * @Description:
 * @date 2023/5/29 16:39
 */
public class DownLoadTask {

    private static final Logger logger = LoggerFactory.getLogger(DownLoadTask.class);

    private final BlockingQueue<String> queue;

    private final Statement stmt;

    private final Connection conn;

    private final String ds;

    public DownLoadTask(BlockingQueue<String> queue, Connection conn , String ds) throws SQLException {
        this.queue = queue;
        this.conn = conn;
        this.stmt = conn.createStatement();
        this.ds = ds;
    }


    public void downLoad(String objectName) throws SQLException {
        OSS ossClient = null;
        try {
            ossClient = OssUtils.getOssClient();
            GetObjectRequest getObjectRequest = new GetObjectRequest(OssUtils.BUCKETNAME, objectName);
            OSSObject ossObject = ossClient.getObject(getObjectRequest);
            InputStream content = ossObject.getObjectContent();
            if (content != null) {
                this.stmt.execute(Common.CREATE_SQL.replaceAll("@ds", ds).replaceAll("@status", "1"));
                BufferedReader reader = new BufferedReader(new InputStreamReader(content));
                while (true) {
                    String line = reader.readLine();
                    if (StringUtils.isBlank(line)) break;
                    JSONObject data = JSONObject.parseObject(line);
                    queue.put(data.getString("sql"));
                }
            }
            if (content != null) {
                content.close();
            }
        } catch (Exception e) {
            logger.error("DownLoad Error Message: {},{}", e.getMessage(), e);
            throw new RuntimeException(e.getMessage());
        } finally {
            // 关闭OSSClient。
            if (ossClient != null) {
                ossClient.shutdown();
            }
            if (stmt != null) {
                stmt.close();
            }
            if (conn != null) {
                conn.close();
            }
        }
    }
}
