package com.winhc.data.task;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author π
 * @Description:
 * @date 2023/5/29 11:41
 */
public class SinkTask extends Thread {
    private static final Logger logger = LoggerFactory.getLogger(SinkTask.class);
    private static int counter = 0;
    private final int id = ++counter;
    private final BlockingQueue<String> queue;
    private final int batchNum;
    private final CountDownLatch latch;
    private final AtomicLong sum = new AtomicLong(0);
    private final Statement stmt;
    private final List<String> batch;
    private final String ds;
    private final Connection connection;


    public SinkTask(BlockingQueue<String> queue, Connection connection, int batchNum, CountDownLatch latch, String ds) throws Exception {
        this.queue = queue;
        this.batchNum = batchNum;
        this.latch = latch;
        this.connection = connection;
        this.stmt = connection.createStatement();
        this.batch = new ArrayList<>();
        this.ds = ds;
    }

    @Override
    public void run() {
        writeData();
        latch.countDown();
    }

    private void writeData() {
        try {
            for (; ; ) {
                String sql = queue.take();
                String QUIT_STR = "QUIT";
                if (QUIT_STR.equalsIgnoreCase(sql)) {
                    break;
                }
                if (StringUtils.isBlank(sql)) {
                    continue;
                }
                batch.add(sql);
                sum.addAndGet(1);
                batch();
            }
            //last batch
            batch();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    private void batch() throws SQLException {
        if (sum.get() % batchNum == 0) {
            if (!batch.isEmpty()) {
                for (String r : batch) {
                    stmt.addBatch(r);
                }
                stmt.executeBatch();
                batch.clear();
            }
            logger.info(String.format("%s %s-%s-total process %s !", ds, Thread.currentThread().getName(), Thread.currentThread().getId(), sum.get()));
        }
    }

    public void close() throws SQLException {
        if (stmt != null) {
            stmt.close();
        }
        if (connection != null) {
            connection.close();
        }
    }

    public long getProcessNum() {
        return sum.get();
    }
}
