package com.winhc.data.utils;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * @author ZhangJi
 * @since 2021-10-22 15:09
 */
public class DateUtils {
    public static final DateTimeFormatter YYYYMMDD = DateTimeFormatter.ofPattern("yyyyMMdd");
    public static final DateTimeFormatter YYYYMMDDHHMMSS = DateTimeFormatter.ofPattern("yyyyMMdd HH:mm:ss");
    public static final DateTimeFormatter YYYY_MM_DDHHMMSS = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    public static final DateTimeFormatter YYYY_MM_DD = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    public static final String FORMAT_YYYYMMDD = "yyyyMMdd";
    public static final String FORMAT_YYYY_MM_DDHHMMSS = "yyyy-MM-dd HH:mm:ss";

    public static final String formatDate(Date date, String pattern) {
        String v = null;
        try {
            if (date == null)
                return null;
            SimpleDateFormat dateFormat = new SimpleDateFormat(pattern);
            v = dateFormat.format(date);
        } catch (Exception e) {
            // do nothing
        }
        return v;
    }

    public static String nowDate() {
        return nowDate(null);
    }

    public static String nowDate(DateTimeFormatter pattern) {
        if (pattern == null) {
            pattern = DateTimeFormatter.ISO_DATE;
        }
        return LocalDateTime.now().format(pattern);
    }

    public static String nowDateTime() {
        return nowDateTime(null);
    }

    public static String nowDateTime(DateTimeFormatter pattern) {
        if (pattern == null) {
            pattern = DateTimeFormatter.ISO_LOCAL_DATE_TIME;
        }
        return LocalDateTime.now().format(pattern);
    }

    public static String getYesterday() {
        return LocalDate.now().plusDays(-1).format(DateTimeFormatter.BASIC_ISO_DATE);
    }

    public static final String FORMAT_YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    public static final String formatDate_YYYY_MM_DD_HH_MM_SS(Date date) {
        String v = null;
        try {
            if (date == null)
                return null;
            SimpleDateFormat dateFormat = new SimpleDateFormat(FORMAT_YYYY_MM_DD_HH_MM_SS);
            v = dateFormat.format(date);
        } catch (Exception e) {
            // do nothing
        }
        return v;
    }

    public static final Date parseDate(String d) {
        Date v = null;
        try {
            if (d == null)
                return null;
            SimpleDateFormat dateFormat = new SimpleDateFormat(FORMAT_YYYY_MM_DDHHMMSS);
            v = dateFormat.parse(d);
        } catch (Exception e) {
            // do nothing
        }
        return v;
    }

    public static final String formatDate_YYYY_MM_DD_HH_MM_SS() {
        return formatDate_YYYY_MM_DD_HH_MM_SS(new Date());
    }

    private static final DateTimeFormatter df = DateTimeFormatter.ofPattern(FORMAT_YYYY_MM_DDHHMMSS)
            .withLocale(Locale.CHINA)
            .withZone(ZoneId.systemDefault());

    public static String getAnyTime(Integer i) {
        Instant instant = Instant.now().minus(i, ChronoUnit.DAYS);
        return df.format(instant);
    }

    public static String longToTime(Long timestamp) {
        if (null == timestamp) return null;
        try {
            SimpleDateFormat format = new SimpleDateFormat(FORMAT_YYYY_MM_DDHHMMSS); //设置格式
            return format.format(timestamp);
        } catch (Exception e) {

        }
        return null;

    }

    public static int getCurrentHours() {
        return new GregorianCalendar().get(Calendar.HOUR_OF_DAY);
    }


    public static void main(String[] args) {
        System.out.println(getYesterday());
        System.out.println(nowDate(YYYYMMDD));
        System.out.println(nowDate(YYYY_MM_DD));
        System.out.println(nowDate(YYYY_MM_DDHHMMSS));
        System.out.println(getCurrentHours());
    }
}
