package com.winhc.data.utils;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.yaml.snakeyaml.Yaml;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/8/31 09:32
 */
public class EnvProperties {
    private static final Logger logger = LoggerFactory.getLogger(EnvProperties.class);

    private static final String fileName = "env.yml";

    private Map<String, String> source;

    public String getValue(String key) {
        return source.getOrDefault(key, null);
    }

    public Map<String, String> getMapByPrefix(String prefix) {
        return source.entrySet().stream().filter(e -> e.getKey().startsWith(prefix)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (e1, e2) -> e1));
    }

    public EnvProperties() {
        this.source = getYmlByFileName(null, null);
    }

    public Map<String, String> getSource() {
        return source;
    }


    /**
     * 根据文件名获取yml的文件内容
     *
     * @return
     */
    private static Map<String, String> getYmlByFileName(String file, Map<String, String> source) {
        if (source == null) {
            source = new HashMap<>();
        }
        if (file == null) {
            //file = fileName;
            return source;
        }
        InputStream in = EnvProperties.class.getClassLoader().getResourceAsStream(file);
        Yaml props = new Yaml();
        Object obj = props.loadAs(in, Map.class);
        Map<String, Object> param = (Map<String, Object>) obj;

        for (Map.Entry<String, Object> entry : param.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            if (val instanceof Map) {
                forEachYaml(source, key, (Map<String, Object>) val);
            } else {
                source.put(key, val.toString());
            }
        }
        return source;
    }

    /**
     * 根据外部文件地址获取yml的文件内容
     *
     * @return
     */
    public static Boolean getYmlByFileNameOut(String filePath, Map<String, String> source) throws FileNotFoundException {
        if (source == null) {
            source = new HashMap<>();
        }
        try {
            logger.info("file path: {}", filePath);
            InputStream in = new FileInputStream(new File(filePath));
            Yaml props = new Yaml();
            Object obj = props.loadAs(in, Map.class);
            Map<String, Object> param = (Map<String, Object>) obj;

            for (Map.Entry<String, Object> entry : param.entrySet()) {
                String key = entry.getKey();
                Object val = entry.getValue();

                if (val instanceof Map) {
                    forEachYaml(source, key, (Map<String, Object>) val);
                } else {
                    source.put(key, val.toString());
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return false;
        }

        return true;
    }

    /**
     * 遍历yml文件，获取map集合
     *
     * @param key_str
     * @param obj
     * @return
     */
    private static Map<String, String> forEachYaml(Map<String, String> source, String key_str, Map<String, Object> obj) {
        for (Map.Entry<String, Object> entry : obj.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            String str_new = "";
            if (StringUtils.isNotBlank(key_str)) {
                str_new = key_str + "." + key;
            } else {
                str_new = key;
            }
            if (val instanceof Map) {
                forEachYaml(source, str_new, (Map<String, Object>) val);
            } else {
                source.put(str_new, val.toString());
            }
        }
        return source;
    }


    public static void main(String[] args) throws IOException {
        EnvProperties envProperties = new EnvProperties();

        String value = envProperties.getValue("winhc.mysql.driverClassName");
        System.out.println(value);
        String value1 = envProperties.getValue("winhc.oss.accessid");
        System.out.println(value1);
    }

}
