package com.winhc.data.utils;


import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static com.winhc.data.utils.Common.*;
import static com.winhc.data.utils.DateUtils.getYesterday;

/**
 * @author π
 * @Description:
 * @date 2022/11/14 18:02
 */

public class OssUtils {

    private static final Logger logger = LoggerFactory.getLogger(OssUtils.class);

    //sj key
/*    public static String ACCESSKEYID = EnvConst.getValue("winhc.oss.accessid");
    public static String ACCESSKEYSECRET = EnvConst.getValue("winhc.oss.accesskey");*/

    public static String ACCESSKEYID = "LTAI5tG3JANdcd6SQPDVSieV";
    public static String ACCESSKEYSECRET = "MFtVO7LvJNTDt6asJLtZhW2S9M5vpQ";

    public static String ENDPOINT = "https://oss-cn-shanghai.aliyuncs.com";
    public static String BUCKETNAME = "data-exchange-sj";


    public static OSS getOssClient() {
        return new OSSClientBuilder().build(ENDPOINT, ACCESSKEYID, ACCESSKEYSECRET);
    }

    public static void saveStatus(String ds, String statusSuffix) {
        OSS ossClient = null;
        try {
            ossClient = getOssClient();
            String targetKey = STATUS_PRE + ds + statusSuffix;
            PutObjectRequest putObjectRequest = new PutObjectRequest(BUCKETNAME, targetKey, new ByteArrayInputStream("".getBytes()));
            getOssClient().putObject(putObjectRequest);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new RuntimeException(e.getMessage());
        } finally {
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }

    }

    public static List<String> listFiles(String statusSuffix) {
        OSS ossClient = getOssClient();
        List<String> list = new ArrayList<>();
        try {
            String nextContinuationToken = null;
            ListObjectsV2Result result;
            do {
                ListObjectsV2Request listObjectsV2Request = new ListObjectsV2Request(BUCKETNAME).withMaxKeys(200).withPrefix(STATUS_PRE);
                listObjectsV2Request.setContinuationToken(nextContinuationToken);
                result = ossClient.listObjectsV2(listObjectsV2Request);
                List<OSSObjectSummary> sums = result.getObjectSummaries();
                for (OSSObjectSummary s : sums) {
                    String key = s.getKey();
                    if (key.contains(statusSuffix)) {
                        list.add(key.replaceAll(STATUS_PRE, "").replaceAll(statusSuffix, ""));
                    }
                }
                nextContinuationToken = result.getNextContinuationToken();
            } while (result.isTruncated());

        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new RuntimeException(e.getMessage());
        } finally {
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }
        return list;
    }

    public static Boolean existsFile(String objectName) {
        OSS ossClient = getOssClient();
        try {
            return ossClient.doesObjectExist(BUCKETNAME, objectName);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new RuntimeException(e.getMessage());
        } finally {
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }
    }

    public static List<String> findTasks(List<String> upload_task) {
        //compare upload and success
        upload_task.addAll(listFiles(STATUS_UPLOAD_SUFFIX));
        List<String> success_task = listFiles(STATUS_SUCCESS_SUFFIX);
        long lastTask = success_task.stream().mapToLong(Long::valueOf).max().orElse(0);
        String yesTask = getYesterday();
        //昨天数据已处理->pass
        if (lastTask == Long.parseLong(getYesterday())) return new ArrayList<>();
        //昨天任务
        /*if (!tasks.contains(yesTask)) {
            tasks.add(yesTask);
        }*/
        return upload_task.stream()
                .filter(x -> !success_task.contains(x) && Long.parseLong(x) > lastTask)
                .distinct().sorted(Comparator.comparing(String::toString)).collect(Collectors.toList());
    }

    public static void main(String[] args) {
        saveStatus("20230528", STATUS_SUCCESS_SUFFIX);
        System.out.println(listFiles(STATUS_SUCCESS_SUFFIX));
        System.out.println(listFiles(STATUS_UPLOAD_SUFFIX));
        System.out.println(existsFile("data/sj/file/20230528.json"));
        System.out.println(findTasks(new ArrayList<>()));
    }
}